/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2008 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors: Graham Kelly <graham@php.net>                               |
  +----------------------------------------------------------------------+
*/
/* $Id: optimizer_debug.h,v 1.10 2009/04/30 03:32:27 myang Exp $ */

#ifndef OPTIMIZE_DEBUG_H
#define OPTIMIZE_DEBUG_H

struct _op_stats_log_row {
	char* message;
	int pass;
	int source_line;
	int opline;
	struct _op_stats_log_row* next;
};


struct _op_stats_basic {
	zend_uint op_row_count;  /* Number of op-rows in the op-array */
	zend_uint var_count;     /* Number of variables in the op-array */
	zend_uint temp_count;    /* Number of temp variables in the op-array */
};


typedef struct _op_stats {
	zend_uint optimization_count;     /* A simple count of the number of optimizations that occured */
	struct _op_stats_basic pre;       /* Stats from before optimization */
	struct _op_stats_basic post;      /* Stats from after optimization */
	struct _op_stats_log_row* log_head;
	struct _op_stats_log_row* log_tail;
} optimizer_stats;




/* Adds 1 to the running total of optimizations performed */
#if OPTIMIZER_STATS_COMPILE
//#  if OPTIMIZER_DEBUG_COMPILE
//#    define OPTIMIZER_STATS_ADD(log) optizer_stats_add_ex(op_array_stats, op, log TSRMLS_CC)
//#  else
#    define OPTIMIZER_STATS_ADD(log) (op_array_stats->optimization_count++) 
//#  endif
#  define OPTIMIZER_STATS_PARAM , optimizer_stats* op_array_stats
#  define OPTIMIZER_STATS_PARAM_PASS ,op_array_stats
#  define OPTIMIZER_STATS_PARAM_DEFAULT ,NULL
#else 
#  define OPTIMIZER_STATS_ADD(log)
#  define OPTIMIZER_STATS_PARAM 
#  define OPTIMIZER_STATS_PARAM_PASS 
#  define OPTIMIZER_STATS_PARAM_DEFAULT 
#endif


#define OPTIMIZER_STATS_ADD_PEEPHOLE(log) \
		OPTIMIZER_STATS_ADD(log);         \

#if OPTIMIZER_STATS_COMPILE
#define OPTIMIZER_STATS_ADD_FCR()             \
		OPTIMIZER_STATS_ADD();
#else
#define OPTIMIZER_STATS_ADD_FCR() do {} while (0)
#endif

void _optimizer_dump_op_array(zend_op_array* op_array, char* file, int line TSRMLS_DC);
void _optimizer_dump_op(zend_op* op, char* file, int line TSRMLS_DC);
void optimizer_dump_cfg(CB* cb, zend_op_array* op_array TSRMLS_DC);
void optimizer_dump_op_array_stats(optimizer_stats* stats, zend_op_array* op_array TSRMLS_DC);
void optimizer_dump_cfg_graphviz(CB* cb, zend_op_array* op_array TSRMLS_DC);
void optimizer_test_dump_op_array(zend_op_array* op_array TSRMLS_DC);
void optimizer_dump_brk_cont_table(zend_op_array* op_array);

#if OPTIMIZER_DEBUG_COMPILE
#  define optimizer_dump_op_array(op_array) _optimizer_dump_op_array(op_array, __FILE__, __LINE__ TSRMLS_CC)
#  define optimizer_dump_op(op) _optimizer_dump_op(op, __FILE__, __LINE__ TSRMLS_CC)
#else
#  define optimizer_dump_op_array(op_array)
#  define optimizer_dump_op(op)
#endif

#endif /* OPTIMIZE_DEBUG_H */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
